@@ -1,7 +1,9 @@
 
-__version__ = "0.1.11"
+__version__ = "1.4.0"
 docs = r"https://alexlib.readthedocs.io/en/latest/"
 git = r"https://github.com/thisismygitrepo/alexlib"
 release_notes = """
-Created Experimental Class
+
+Cleaned deeplearning_torch module. Removed Assert module.
+
 """
