@@ -291,7 +291,7 @@ class BaseModel(ABC):
     * :func:`BaseModel.predict` expects a processed input, uese infer and does postprocessing.
     * :func:`BaseModel.predict_from_s` reads, preprocess, then uses predict method.
     * :func:`BseModel.evaluate` Expects processed input and internally calls infer and postprocess methods.
-    
+
     """
 
     @abstractmethod
@@ -404,11 +404,12 @@ class BaseModel(ABC):
         inferred = self.infer(x)
         return self.postprocess(inferred, **kwargs)
 
-    def deduce(self, obj_s, viz=True, **kwargs):
-        preprocessed = np.concatenate([self.preprocess(obj) for obj in obj_s], axis=0)
+    def deduce(self, obj, viz=True, **kwargs):
+        """Assumes that contents of the object are in the form of a batch."""
+        preprocessed = self.preprocess(obj)
         prediction = self.infer(preprocessed)
         postprocessed = self.postprocess(prediction, **kwargs)
-        result = tb.Struct(input=obj_s, preprocessed=preprocessed, prediction=prediction, postprocessed=postprocessed)
+        result = tb.Struct(input=obj, preprocessed=preprocessed, prediction=prediction, postprocessed=postprocessed)
         if viz:
             self.viz(postprocessed, **kwargs)
         return result
@@ -488,7 +489,7 @@ class BaseModel(ABC):
         name = directory.glob('*.data*').__next__().__str__().split('.data')[0]
         self.model.load_weights(name)  # requires path to file name.
 
-    def save_class(self, weights_only=True, save_sourcecode=True, version='0', itself=False, **kwargs):
+    def save_class(self, weights_only=True, version='0', itself=False, **kwargs):
         """Simply saves everything:
 
         1. Hparams
@@ -500,25 +501,19 @@ class BaseModel(ABC):
         :return:
 
         """
-        _ = kwargs
-        self.hp.save_pickle(itself=itself)
-        self.data.save_pickle(itself=itself)
+        self.hp.save_pickle(itself=itself)  # goes into the meta folder.
+        self.data.save_pickle(itself=itself)  # goes into the meta folder.
+        self.history.save_npy(path=self.hp.save_dir / 'metadata/history.npy', itself=True)  # goes to meta folder
 
+        # model save goes into data folder.
         save_dir = self.hp.save_dir.joinpath(f'{"weights" if weights_only else "model"}_save_v{version}').create()
         if weights_only:
             self.save_weights(save_dir)
         else:
             self.save_model(save_dir)
 
-        # Saving wrapper_class
-        import inspect
-        code_string = inspect.getsource(self.__class__)
-        meta_dir = tb.P(self.hp.save_dir).joinpath('metadata').create()
-        meta_dir.joinpath('model_arch.txt').write_text(code_string)
-        if save_sourcecode:
-            tb.P(inspect.getmodule(self).__file__).zip(op_path=meta_dir / "source_code")
-
-        self.history.save_npy(path=meta_dir.joinpath('history.npy'))
+        # Saving wrapper_class and model architecture in the main folder:
+        tb.Experimental.generate_readme(self.hp.save_dir, obj=self.__class__, **kwargs)
         print(f'Model class saved successfully!, check out: \n {self.hp.save_dir.as_uri()}')
 
     @classmethod
@@ -542,7 +537,7 @@ class BaseModel(ABC):
             data_path = path / DataReader.subpath
             data_obj = tb.Read.pickle(data_path) if data_path.exists() else None
         model_obj = cls(hp_obj, data_obj)
-        model_obj.load_weights(path.myglob('*_save_*')[0])
+        model_obj.load_weights(path.search('*_save_*')[0])
         history = path / "metadata/history.npy"
         model_obj.history = tb.List.from_saved(history) if history.exists() else tb.List()
         print(f"Class {model_obj.__class__} Loaded Successfully.")
@@ -553,7 +548,7 @@ class BaseModel(ABC):
         path = tb.P(path)
         data_obj = DataReader.from_saved(path)
         hp_obj = HyperParam.from_saved(path)
-        model_obj = cls.load_model(path.myglob('*_save_*')[0])  # static method.
+        model_obj = cls.load_model(path.search('*_save_*')[0])  # static method.
         tmp = cls.__init__
 
         def initializer(self, hp_object, data_object, model_object_):
@@ -575,10 +570,17 @@ class BaseModel(ABC):
             print("==============================")
 
     def plot_model(self, **kwargs):
+        """
+        .. note:: Functionally or Sequentually built models are much more powerful than Subclassed models.
+            They are faster, have more features, can be plotted, serialized, correspond to computational graphs etc.
+            Alternative visualization is via tf2onnx then Netron.
+        """
         import tensorflow as tf
         tf.keras.utils.plot_model(self.model, to_file=self.hp.save_dir / 'model_plot.png',
-                                  show_shapes=True, show_layer_names=True, **kwargs)
-        print('Successfully plotted the model')
+                                  show_shapes=True, show_layer_names=True, show_dtype=True,
+                                  expand_nested=True,
+                                  dpi=150, **kwargs)
+        print('Successfully plotted the model, check out \n', (self.hp.save_dir / 'model_plot.png').as_uri())
 
     def build(self, shape=None, verbose=True):
         """ Building has two main uses.
@@ -640,14 +642,14 @@ class Ensemble(tb.Base):
 
     @classmethod
     def from_saved_models(cls, parent_dir, model_class):
-        obj = cls(model_class=model_class, path=parent_dir, size=len(tb.P(parent_dir).myglob('*__model__*')))
-        obj.models = tb.P(parent_dir).myglob('*__model__*').apply(model_class.from_class_model)
+        obj = cls(model_class=model_class, path=parent_dir, size=len(tb.P(parent_dir).search('*__model__*')))
+        obj.models = tb.P(parent_dir).search('*__model__*').apply(model_class.from_class_model)
         return obj
 
     @classmethod
     def from_saved_weights(cls, parent_dir, model_class):
-        obj = cls(model_class=model_class, path=parent_dir, size=len(tb.P(parent_dir).myglob('*__model__*')))
-        obj.models = tb.P(parent_dir).myglob('*__model__*').apply(model_class.from_class_weights)
+        obj = cls(model_class=model_class, path=parent_dir, size=len(tb.P(parent_dir).search('*__model__*')))
+        obj.models = tb.P(parent_dir).search('*__model__*').apply(model_class.from_class_weights)
         return obj
 
     def fit(self, shuffle_train_test=True, save=True, **kwargs):
@@ -668,29 +670,44 @@ class Ensemble(tb.Base):
         pass
 
 
-def get_mean_max_error(tf):
-    """
-    For Tensorflow
-    """
+class Losses:
+    @staticmethod
+    def get_log_square_loss_class():
+        import tensorflow as tf
+        class LogSquareLoss(tf.keras.losses.Loss):
+            def __init__(self, *args, **kwargs):
+                super().__init__(*args, **kwargs)
+                self.name = "LogSquareLoss"
 
-    class MeanMaxError(tf.keras.metrics.Metric):
-        def __init__(self, name='MeanMaximumError', **kwargs):
-            super(MeanMaxError, self).__init__(name=name, **kwargs)
-            self.mme = self.add_weight(name='mme', initializer='zeros')
-            self.__name__ = name
+            def call(self, y_true, y_pred):
+                factor = (20 / tf.math.log(tf.convert_to_tensor(10.0, dtype=y_pred.dtype)))
+                return factor * tf.math.log(tf.reduce_mean(abs(y_true - y_pred)))
+        return LogSquareLoss
 
-        def update_state(self, y_true, y_pred, sample_weight=None):
-            if sample_weight is None:
-                sample_weight = 1.0
-            self.mme.assign(tf.reduce_mean(tf.reduce_max(sample_weight * tf.abs(y_pred - y_true), axis=1)))
+    @staticmethod
+    def get_mean_max_error(tf):
+        """
+        For Tensorflow
+        """
 
-        def result(self):
-            return self.mme
+        class MeanMaxError(tf.keras.metrics.Metric):
+            def __init__(self, name='MeanMaximumError', **kwargs):
+                super(MeanMaxError, self).__init__(name=name, **kwargs)
+                self.mme = self.add_weight(name='mme', initializer='zeros')
+                self.__name__ = name
 
-        def reset_states(self):
-            self.mme.assign(0.0)
+            def update_state(self, y_true, y_pred, sample_weight=None):
+                if sample_weight is None:
+                    sample_weight = 1.0
+                self.mme.assign(tf.reduce_mean(tf.reduce_max(sample_weight * tf.abs(y_pred - y_true), axis=1)))
 
-    return MeanMaxError
+            def result(self):
+                return self.mme
+
+            def reset_states(self):
+                self.mme.assign(0.0)
+
+        return MeanMaxError
 
 
 class HPTuning:
